@@ -17,14 +17,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """An experimental docking implementation.
-    
-# Todo
- * API cleanup
- 
+
 """
 
 import gtk
 import gobject
+import pango
 from lxml import etree
 from cStringIO import StringIO
 import cairo
@@ -124,6 +122,12 @@ class DockBar(gtk.HBox):
         btn.set_data("item", item)
         lbl = gtk.Label(item.label)
         btn.set_label_widget(lbl)
+        if item.icon:
+            it = gtk.icon_theme_get_default()
+            pb = it.load_icon(item.icon, gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG)
+            i = gtk.Image()
+            i.set_from_pixbuf(pb)
+            btn.set_icon_widget(i)
         if self.pos in [gtk.POS_LEFT, gtk.POS_RIGHT]:
             lbl.set_angle(270)
         self.bar.insert(btn, -1)
@@ -144,6 +148,7 @@ class DockNotebook(gtk.Notebook):
         self.__gobject_init__()
         self.set_tab_pos(gtk.POS_BOTTOM)
         self.set_scrollable(True)
+        self.set_homogeneous_tabs(False)
         self.popup_enable()
         self.connect("page-removed", self.on_page_count_changed)
         self.connect("page-added", self.on_page_count_changed)
@@ -165,13 +170,13 @@ class DockGroup(gtk.EventBox):
         self.vpaned1 = gtk.VPaned()
         self.add(self.vpaned1)
         self.vpaned2 = gtk.VPaned()
-        self.vpaned1.add2(self.vpaned2)
+        self.vpaned1.pack2(self.vpaned2)
         self.hpaned1 = gtk.HPaned()
-        self.vpaned2.add1(self.hpaned1)
+        self.vpaned2.pack1(self.hpaned1)
         self.hpaned2 = gtk.HPaned()
-        self.hpaned1.add2(self.hpaned2)
+        self.hpaned1.pack2(self.hpaned2)
         self.notebook = DockNotebook()
-        self.hpaned2.add1(self.notebook)
+        self.hpaned2.pack1(self.notebook)
         self.group_top = None
         self.group_bottom = None
         self.group_left = None
@@ -218,6 +223,9 @@ class DockGroup(gtk.EventBox):
             if x >= gx and x <= gx+group.allocation.width \
             and y >= gy and y <= gy+group.allocation.height:
                 return group.find_group_by_position(dock_item, x, y)
+        # no subgroups, if it's in notebook there's nothing to do
+        if dock_item in self.notebook.get_children():
+            return self, None
         gx, gy = self.window.get_origin()
         if y <= gy+self.allocation.height/3.0 \
         and dock_item.behavior & DOCK_ITEM_BEH_NEVER_VERTICAL == 0:
@@ -242,16 +250,20 @@ class DockGroup(gtk.EventBox):
             pos = pos
         if pos == None:
             item.status = DOCK_ITEM_STATUS_DOCKED
-            self.notebook.append_page(item, gtk.Label(item.label))
-            self.notebook.set_current_page(self.notebook.get_n_pages())
+            lbl = gtk.Label(item.label)
+            lbl.set_single_line_mode(True)
+            lbl.set_ellipsize(pango.ELLIPSIZE_END)
+            lbl.set_max_width_chars(30)
+            self.notebook.append_page(item, lbl)
             self.notebook.set_tab_reorderable(item, True)
+            gobject.idle_add(self.notebook.set_current_page, self.notebook.get_n_pages()-1)
         elif pos == gtk.POS_TOP:
             if self.group_top:
                 self.group_top.add_item(item, pos=None)
             else:
                 self.group_top = DockGroup(self.dock, "top")
                 self.group_top.add_item(item, pos=None)
-                self.vpaned1.add1(self.group_top)
+                self.vpaned1.pack1(self.group_top)
                 self.group_top.connect("destroy", self.on_group_destroyed, "group_top")
         elif pos == gtk.POS_BOTTOM:
             if self.group_bottom:
@@ -259,7 +271,7 @@ class DockGroup(gtk.EventBox):
             else:
                 self.group_bottom = DockGroup(self.dock, "bottom")
                 self.group_bottom.add_item(item, pos=None)
-                self.vpaned2.add2(self.group_bottom)
+                self.vpaned2.pack2(self.group_bottom, False, False)
                 self.group_bottom.connect("destroy", self.on_group_destroyed, "group_bottom")
         elif pos == gtk.POS_LEFT:
             if self.group_left:
@@ -267,7 +279,7 @@ class DockGroup(gtk.EventBox):
             else:
                 self.group_left = DockGroup(self.dock, "left")
                 self.group_left.add_item(item, pos=None)
-                self.hpaned1.add1(self.group_left)
+                self.hpaned1.pack1(self.group_left)
                 self.group_left.connect("destroy", self.on_group_destroyed, "group_left")
         elif pos == gtk.POS_RIGHT:
             if self.group_right:
@@ -275,7 +287,7 @@ class DockGroup(gtk.EventBox):
             else:
                 self.group_right = DockGroup(self.dock, "right")
                 self.group_right.add_item(item, pos=None)
-                self.hpaned2.add2(self.group_right)
+                self.hpaned2.pack2(self.group_right)
                 self.group_right.connect("destroy", self.on_group_destroyed, "group_right")
         self.show_all()
         self.dock.update_views()
@@ -420,8 +432,13 @@ class DockItem(gtk.VBox):
         self.header.modify_bg(gtk.STATE_NORMAL, style.bg[gtk.STATE_ACTIVE])
         hbox = gtk.HBox()
         hbox.set_spacing(3)
+        hbox.set_border_width(2)
         if self.icon:
-            i = gtk.image_new_from_stock(self.icon, gtk.ICON_SIZE_MENU)
+            #i = gtk.image_new_from_stock(self.icon, gtk.ICON_SIZE_MENU)
+            it = gtk.icon_theme_get_default()
+            pb = it.load_icon(self.icon, gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG)
+            i = gtk.Image()
+            i.set_from_pixbuf(pb)
             hbox.pack_start(i, False, False)
         a = gtk.Alignment(0, 0.5)
         a.add(gtk.Label(self.label))
@@ -432,6 +449,7 @@ class DockItem(gtk.VBox):
                 btn.set_relief(gtk.RELIEF_NONE)
                 btn.set_property("can-focus", False)
                 btn.set_size_request(17, 17)
+                btn.set_border_width(0)
                 btn.set_image(gtk.image_new_from_stock("gtk-leave-fullscreen", gtk.ICON_SIZE_MENU))
                 btn.connect("clicked", self.on_send_to_bar)
                 hbox.pack_start(btn, False, False)
@@ -458,6 +476,7 @@ class DockItem(gtk.VBox):
         else:
             self.pack_start(self.widget, True, True)
         self.dock.items.append(self)
+        self.widget.connect("destroy", self.on_widget_destroy)
         self.connect("destroy", self.on_destroy)
             
     def on_close(self, *args):
@@ -505,6 +524,9 @@ class DockItem(gtk.VBox):
     def on_header_realize(self, header, *args):
         header.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
         
+    def on_widget_destroy(self, *args):
+        gobject.idle_add(self.destroy)
+        
     def send_to_bar(self):
         self.hide()
         parent = self.get_parent()
@@ -695,6 +717,7 @@ class FloatingWindow(gtk.Window):
     def __init__(self, item):
         gtk.Window.__init__(self)
         self.item = item
+        self.set_title(item.label)
         self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
         self.item.reparent(self)
         self.connect("remove", self.on_remove)
